#!/bin/sh 

# Signal handler.
SignalTrap()
{
	echo "Received signal.  Update aborting."
	exit 1
}

# Set Signal Trap
trap SignalTrap INT QUIT TERM ABRT STOP

# Change to "True" to output debugging information to root of USB
DEBUGGING="True"
# CONSTANTS
PART_NUMBER=""
#Hardware Variants
HW_OVERLAY="OVERLAY"
HW_30="30"
HW_31="31"
HW_B460="B460"
HW_32="32"
#Nav variants
NAV="NAV"
NNAV="NNAV"
SCRIPT_VERSION=""
#Configuration Variables
LANG=""
NOW=$(date +"%m_%d_%Y")
#Script Arguments
UPDATE_ARGUMENT=1
UP_TO_DATE_ARGUMENT=2
TRANSACTION_ONE=1
TRANSACTION_TWO=2


function getLanguage
{
	chmod 755 /tmp/getLanguage.sh
	LANG=$(/tmp/getLanguage.sh)
	logMessage "Language detected: ${LANG}"
	#English or French Canadian
	if [[ $LANG != "ENU" && $LANG != "FRC" ]]
	then
		LANG="ENU"
		logMessage "Language defaulted to ENU"
	fi
}

############################################MESSAGES TO DISPLAY################################################
function upToDateMessage
{
	#"Module is up to date."
	logMessage "Module is up to date"

	cmp -s /fs/usb0/autoinstall.lst /fs/usb0/SyncMyRide/autoInstalls/autoinstall.lst
	# Does autoinstall have anything else added that we don't want to run? If not then we can display message and exit
	if [[ $? -eq 0 ]]
	then
		sh /fs/rwdata/storage/updater/status_message.sh $UP_TO_DATE_ARGUMENT &
		exit 0
	fi

	# module is up to date but root autoinstall is not what it should be so we restore the autoInstall and restart the module
	restoreBaseAutoinstall
	restartMessageNoVersion
	killProgram
	shutdown
}

function restartMessage
{
	#"Please stand by. Module will update from "XXX" to "YYY". Multiple reboots expected."

	VERSION=$(grep "SYNCGen3.0" /fs/mp/Version.inf | awk -F_ '{ print $2 }')
	MAJOR=$(echo $VERSION | awk -F. '{ print $1 }')
	if [[ $MAJOR -eq 1 ]]
	then
		MY16_displayMessage
	else
		MESSAGE=$(grep $LANG /fs/rwdata/storage/updater/languages.txt | awk -F= '{ print $2 }' | awk -F+ '{ print $3 }' | sed "s/XXX/$VERSION/g" | sed "s/YYY/$VERSION_NUMBER/g")
		displayMessage "$MESSAGE"
		# sleep so the viewer has time to view the message
		sleep 7
	fi
}

function restartMessageNoVersion
{
	#"Please stand by. System restarting."

	VERSION=$(grep "SYNCGen3.0" /fs/mp/Version.inf | awk -F_ '{ print $2 }')
	MAJOR=$(echo $VERSION | awk -F. '{ print $1 }')
	if [[ $MAJOR -eq 1 ]]
	then
		MY16_displayMessage
	else
		MESSAGE=$(grep $LANG /fs/rwdata/storage/updater/languages.txt | awk -F= '{ print $2 }' | awk -F+ '{ print $7 }')
		displayMessage "$MESSAGE"
		# sleep so the viewer has time to view the message
		sleep 5
	fi
}

function corruptMessage
{
	#"Corrupt USB tool detected. Please update USB using dealership tool."

	# check to see if module is MY16
	VERSION=$(grep "SYNCGen3.0" /fs/mp/Version.inf | awk -F_ '{ print $2 }')
	MAJOR=$(echo $VERSION | awk -F. '{ print $1 }')
	if [[ $MAJOR -eq 1 ]]
	then
		MY16_errorMessage
	else
		MESSAGE=$(grep $LANG /fs/rwdata/storage/updater/languages.txt | awk -F= '{ print $2 }' | awk -F+ '{ print $4 }')
		displayMessage "$MESSAGE"
	fi
	safeExit
}

function missingFileMessage
{
	#"Required file does not exist"

	# check to see if module is MY16
	VERSION=$(grep "SYNCGen3.0" /fs/mp/Version.inf | awk -F_ '{ print $2 }')
	MAJOR=$(echo $VERSION | awk -F. '{ print $1 }')
	if [[ $MAJOR -eq 1 ]]
	then
		MY16_errorMessage
	else
		MESSAGE=$(grep $LANG /fs/rwdata/storage/updater/languages.txt | awk -F= '{ print $2 }' | awk -F+ '{ print $5 }')
		displayMessage "$MESSAGE"
	fi
	safeExit
}

function screenDetectionMessage
{
	#"Issue detecting screen. Cannot determine correct software package."
	MESSAGE=$(grep $LANG /fs/rwdata/storage/updater/languages.txt | awk -F= '{ print $2 }' | awk -F+ '{ print $6 }')
	displayMessage "$MESSAGE"
	safeExit
}

function leaseExpiredMessage
{
	# USB lease has expired
	# check to see if module is MY16
	MAJOR=$(grep "SYNCGen3.0" /fs/mp/Version.inf | awk -F_ '{ print $2 }' | awk -F. '{ print $1 }')
	if [[ $MAJOR -eq 1 ]]
	then
		MY16_displayPNG "lease_expired_message"
	else
		MESSAGE=$(grep $LANG /fs/rwdata/storage/updater/languages.txt | awk -F= '{ print $2 }' | awk -F+ '{ print $10 }')
		displayMessage "$MESSAGE"
	fi
	
	safeExit
}

function leaseAlmostExpiredMessage
{
	# USB lease has almost expired
	MESSAGE=$(grep $LANG /fs/rwdata/storage/updater/languages.txt | awk -F= '{ print $2 }' | awk -F+ '{ print $11 }')
	displayMessage "$MESSAGE"
}

function leaseAbsentMessage
{
	# paste the link onto a .txt file on the USB
	echo "$DESKTOP_APPLICATION">/fs/usb0/desktop_app_link.txt

	# check to see if module is MY16
	MAJOR=$(grep "SYNCGen3.0" /fs/mp/Version.inf | awk -F_ '{ print $2 }' | awk -F. '{ print $1 }')
	if [[ $MAJOR -eq 1 ]]
	then
		MY16_displayPNG "lease_absent_message"
	else
		# USB lease is not present on the USB
		MESSAGE=$(grep $LANG /fs/rwdata/storage/updater/languages.txt | awk -F= '{ print $2 }' | awk -F+ '{ print $12 }')
		displayMessage "$MESSAGE"
	fi

	safeExit
}

function displayMessage
{
	#$1 is the message to display

	HEADER=$(grep $LANG /fs/rwdata/storage/updater/languages.txt | awk -F= '{ print $2 }' | awk -F+ '{ print $1 }')
	POPUP_TXT_FILE=/fs/rwdata/storage/updater/pop_up.txt
	echo "${HEADER} ${SCRIPT_VERSION}: ${1}" > $POPUP_TXT_FILE
	chmod 755 /fs/rwdata/storage/updater/utserviceutility
	/fs/rwdata/storage/updater/utserviceutility popup $POPUP_TXT_FILE
	rm -f $POPUP_TXT_FILE
}

function MY16_displayPNG
{
	slay APP_SUM
	slay NAV_Manager
	slay fordhmi
	slay HMI_AL
	display_image -file=/fs/rwdata/storage/updater/my16_images/${1}_${LANG}.png -display=2 &

	find /fs/images/ivsu_* -name "*.lst*" -remove!
	find /fs/images/ivsu_* -name "*.tar.gz" -remove!
	sync
	sync

	sleep 9
}

function MY16_displayMessage
{
	MY16_displayPNG "updater_message"
	shutdown
	exit 0
}

function MY16_errorMessage
{
	MY16_displayPNG "invalid_usb"
	exit 0
}

##########################################END MESSAGES TO DISPLAY##############################################

function logMessage
{
	if [[ $DEBUGGING = "True" ]]
	then
		echo "$1">>/fs/usb0/logging.txt
	fi
}

function appendCorePackages
{
	if [[ "$PACKAGE_VOICE" != "$CURRENT_VOICE" && "$VOICE" != "" ]]
	then
		appendAutoinstall "$VOICE"
	fi
	if [[ "$PACKAGE_APPS" != "$CURRENT_APPS" && "$APPS" != "" ]]
	then
		appendAutoinstall "$APPS"
	fi
	if [[ "$PACKAGE_GRACENOTES" != "$CURRENT_GRACENOTES" && "$GRACENOTES" != "" ]]
	then
		appendAutoinstall "$GRACENOTES"
	fi
	if [[ "$PACKAGE_DAB" != "$CURRENT_DAB" && "$DAB" != "" ]]
	then
		appendAutoinstall "$DAB"
	fi
}

function generateAutoInstall
{
	#1 is hardware type
	#2 is screen size
	#3 is region
	#4 is nav
	#5 is update version
	logMessage "Entering generateAutoInstall"
	HARDWARE=$1
	if [[ $HARDWARE = "$HW_OVERLAY" ]]
	then
		HARDWARE=$HW_30
	fi
	PACKAGES=$(grep "${5}_${HARDWARE}_${2}_${3}_${4}" /fs/rwdata/storage/updater/packages.lst | awk -F= '{ print $2 }')
	VOICE=$(echo $PACKAGES | awk -F, '{ print $1 }' | awk -F. '{ print $1 }')
	APPS=$(echo $PACKAGES | awk -F, '{ print $2 }' | awk -F. '{ print $1 }')
	GRACENOTES=$(echo $PACKAGES | awk -F, '{ print $3 }' | awk -F. '{ print $1 }')
	DAB=$(echo $PACKAGES | awk -F, '{ print $4 }' | awk -F. '{ print $1 }')

	PACKAGE_VOICE=$(echo $VOICE | awk -F_ '{ print $1 }')
	PACKAGE_APPS=$(echo $APPS | awk -F_ '{ print $1 }')
	PACKAGE_GRACENOTES=$(echo $GRACENOTES | awk -F_ '{ print $1 }')
	PACKAGE_DAB=$(echo $DAB | awk -F_ '{ print $1 }')

	CURRENT_VOICE=$(grep "Ford Part Number" /fs/rwdata/.Version/*391* | awk -F= '{ print $2 }' | sed 's/ //g' | awk -F_ '{ print $1 }' | tr -d '\r')
	CURRENT_APPS=$(grep "Ford Part Number" /fs/rwdata/.Version/*381* | awk -F= '{ print $2 }' | sed 's/ //g' | awk -F_ '{ print $1 }' | tr -d '\r')
	CURRENT_GRACENOTES=$(grep "Ford Part Number" /fs/rwdata/.Version/*423* | awk -F= '{ print $2 }' | sed 's/ //g' | awk -F_ '{ print $1 }' | tr -d '\r')
	CURRENT_DAB$(grep "Ford Part Number" /fs/rwdata/.Version/*658* | awk -F= '{ print $2 }' | sed 's/ //g' | awk -F_ '{ print $1 }' | tr -d '\r')
	logMessage "Module Current Voice Package: ${CURRENT_VOICE}"
	logMessage "Updating to: ${VOICE}"
	logMessage "Module Current Apps Package: ${CURRENT_APPS}"
	logMessage "Updating to: ${APPS}"
	logMessage "Module Current Gracenotes Package: ${CURRENT_GRACENOTES}"
	logMessage "Updating to: ${GRACENOTES}"
	logMessage "Module Current Database Package: ${CURRENT_DAB}"
	logMessage "Updating to: ${DAB}"
	logMessage "Interrogator: ${INTERROGATOR}"

	FIRST_HEADER="[SYNCGen3.0_${5}_PRODUCT]"

	ITEM_INDEX=1
	NEU_NNAV=0

	if [[ $1 != "$HW_OVERLAY" ]]
	then
		if [[ ( $1 = "$HW_30" || $1 = "$HW_31" ) && ( $2 -eq 6 || $2 -eq 8 ) && $3 = "NEU" ]]
		then
			# 3.0/3.1 6/8 NEU module
			NEU_NNAV=1
			ITEM1="GB5T-14G386-SC"
			echo "${FIRST_HEADER}\n">/fs/rwdata/storage/updater/autoinstall.lst
			appendAutoinstall "$VCA" "SYNC3 VCA Utility"
			appendAutoinstall "${PART_NUMBER}" "SYNC 3 Utility"
			appendAutoinstall "$ITEM1"
			appendAutoinstall "$VOICE"
			echo "Options = AutoInstall, Silent\n">>/fs/rwdata/storage/updater/autoinstall.lst
			ITEM_INDEX=1
			echo "[SYNCGen3.0_ALL]\n">>/fs/rwdata/storage/updater/autoinstall.lst
			appendAutoinstall "${PART_NUMBER}" "SYNC 3 Utility"
			appendAutoinstall "$APPS"
			appendAutoinstall "$GRACENOTES"
			appendAutoinstall "$INTERROGATOR"
			echo "Options = AutoInstall, Include, Transaction\n">>/fs/rwdata/storage/updater/autoinstall.lst
		else
			echo "${FIRST_HEADER}\n">/fs/rwdata/storage/updater/autoinstall.lst
			appendAutoinstall "$VCA" "SYNC3 VCA Utility"
			appendAutoinstall "${PART_NUMBER}" "SYNC 3 Utility"
			echo "Options = AutoInstall, Silent\n">>/fs/rwdata/storage/updater/autoinstall.lst
			ITEM_INDEX=1
			echo "[SYNCGen3.0_ALL]\n">>/fs/rwdata/storage/updater/autoinstall.lst
			appendAutoinstall "${PART_NUMBER}" "SYNC 3 Utility"
			appendCorePackages
			appendAutoinstall "$INTERROGATOR"
			echo "Options = AutoInstall, Include, Transaction\n">>/fs/rwdata/storage/updater/autoinstall.lst
		fi
	else
		ITEM1="HB5T-14G386-AHH"
		ITEM2="HB5T-14G387-AAB"
		echo "${FIRST_HEADER}\n">/fs/rwdata/storage/updater/autoinstall.lst
		if [[ $3 != "NEU" ]]
		then
			appendAutoinstall "$VCA" "SYNC3 VCA Utility"
			appendAutoinstall "${PART_NUMBER}" "SYNC 3 Utility"
			echo "Options = AutoInstall, Silent\n">>/fs/rwdata/storage/updater/autoinstall.lst
			ITEM_INDEX=1
			echo "[SYNCGen3.0_ALL]\n">>/fs/rwdata/storage/updater/autoinstall.lst
			appendAutoinstall "${PART_NUMBER}" "SYNC 3 Utility"
			appendAutoinstall "$ITEM1"
			appendAutoinstall "${PART_NUMBER}_duplicate" "Status Indicator"
			appendCorePackages
			appendAutoinstall "$ITEM2"
			appendAutoinstall "$INTERROGATOR"
			echo "Options = AutoInstall, Include, Transaction\n">>/fs/rwdata/storage/updater/autoinstall.lst
		else
			NEU_NNAV=1
			ITEM3="GB5T-14G386-SC"
			appendAutoinstall "$VCA" "SYNC3 VCA Utility"
			appendAutoinstall "${PART_NUMBER}" "SYNC 3 Utility"
			appendAutoinstall "$ITEM1"
			appendAutoinstall "${PART_NUMBER}_duplicate" "Status Indicator"
			appendAutoinstall "$ITEM3"
			appendAutoinstall "$VOICE"
			echo "Options = AutoInstall, Silent\n">>/fs/rwdata/storage/updater/autoinstall.lst
			ITEM_INDEX=1
			echo "[SYNCGen3.0_ALL]\n">>/fs/rwdata/storage/updater/autoinstall.lst
			appendAutoinstall "${PART_NUMBER}" "SYNC 3 Utility"
			appendAutoinstall "$ITEM2"
			appendAutoinstall "$GRACENOTES"
			appendAutoinstall "$APPS"
			appendAutoinstall "$INTERROGATOR"
			echo "Options = AutoInstall, Include, Transaction\n">>/fs/rwdata/storage/updater/autoinstall.lst
		fi
	fi

	cmp -s /fs/usb0/autoinstall.lst /fs/usb0/SyncMyRide/autoInstalls/autoinstall.lst
	REBOOT=$?
	rm /fs/usb0/autoinstall.lst
	cp /fs/rwdata/storage/updater/autoinstall.lst /fs/usb0/autoinstall.lst
	keyGenerator
	# find /fs/images/ivsu_* -name "*.lst*" -remove!
	restartMessage
	killProgramWipeAndRestart "$1"
	exit 0
}

function extractTar
{
	logMessage "Entering extractTar"
	# check if directory already exists
	if [[ -e /fs/rwdata/storage/updater ]]
	then
		rm -r /fs/rwdata/storage/updater
	fi
	# create fresh directory and populate
	mkdir /fs/rwdata/storage/updater
	cd /fs/rwdata/storage/updater
	tar -xf /tmp/universalUpdater.tar
	# copy getlanguage.sh to /updater for use in status_message.sh
	cp /tmp/getLanguage.sh /fs/rwdata/storage/updater
}

function variableAssign
{
	logMessage "Entering variableAssign"
	# verify packages.lst signature
	chmod 755 /fs/rwdata/storage/updater/verifyPackage
	/fs/rwdata/storage/updater/verifyPackage
	success=$?
	if [[ $success -ne 0 ]]
	then
		logMessage "Could not verify package"
		corruptMessage
		safeExit
	fi
	# reads the up to date versions from SyncMyRide/autoInstalls/packages.lst
	logMessage "Successfully verified packages"

	PACKAGES=$(find /fs/rwdata/storage/updater/packages.lst)
	if [[ "$PACKAGES" = "" ]]
	then
		corruptMessage
		safeExit
	fi
	PART_NUMBER=$(grep "PART_NUMBER" $PACKAGES | awk -F= '{ print $2 }')
	SCRIPT_VERSION=$(grep "SCRIPT_VERSION" $PACKAGES | awk -F= '{ print $2 }')
	INTERROGATOR=$(grep "INTERROGATOR" $PACKAGES | awk -F= '{ print $2 }')
	VCA=$(grep "VCA_UTILITY" $PACKAGES | awk -F= '{ print $2 }')
	DESKTOP_APPLICATION=$(grep "DESKTOP_APPLICATION" $PACKAGES | awk -F= '{ print $2 }')

	logMessage "PART_NUMBER = ${PART_NUMBER}"
	logMessage "SCRIPT_VERSION = ${SCRIPT_VERSION}"
	logMessage "INTERROGATOR = ${INTERROGATOR}"
	logMessage "VCA = ${VCA}"
}

function safeExit
{
	logMessage "Entering safeExit"
	# Remove Cache
	rm /fs/rwdata/storage/region.txt
	# This function is called when we want to exit early but aren't sure the status of autoinstall
	cmp -s /fs/usb0/autoinstall.lst /fs/usb0/SyncMyRide/autoInstalls/autoinstall.lst
	UNSAFE=$?
	# Does auto install have anything else added that we don't want to run? If not then we can just exit
	if [[ $UNSAFE -eq 0 ]]
	then
		logMessage "Safe to exit"
		exit 0
	fi

	restoreBaseAutoinstall
	rm -r /fs/rwdata/storage/updater
	killProgram
	exit 0
}

function invalidKey
{
	VERSION=$(grep "SYNCGen3.0" /fs/mp/Version.inf | awk -F_ '{ print $2 }')
	logMessage "Current version: $VERSION"
	extractTar
	variableAssign
	leaseCheck
	MY16_Module $VERSION
	checkOverlay $VERSION
	#checkGT $VERSION
	hwCheck "$VERSION"
}

function killProgram
{
	logMessage "Killing program and wiping"
	slay APP_SUM
	find /fs/images/ivsu_* -name "*.lst*" -remove!
	find /fs/images/ivsu_* -name "*.tar.gz" -remove!
	sync
	sync
}

function killProgramWipeAndRestart
{
	# $1 is hardware variant
	killProgram

	# check to see if root autoinstall.lst was the same as the autoinstall.lst in the autoInstalls directory
	# If root autoinstall was different and we have NEU NNAV module, then we need to restart the module
	if [[ $REBOOT -ne 0 && $NEU_NNAV -ne 0 ]]
	then
		logMessage "Restarting Sync"
		shutdown
	else
		logMessage "Restarting program"
		tar tf /fs/usb0/SyncMyRide/${PART_NUMBER}.tar.gz | awk '{print "/tmp/"$1}' | xargs rm
		/fs/mp/apps/APP_SUM &
	fi
}

function restoreBaseAutoinstall
{
	logMessage "Restoring base autoinstall.lst"
	rm /fs/usb0/autoinstall.lst
	cp /fs/usb0/SyncMyRide/autoInstalls/autoinstall.lst /fs/usb0/autoinstall.lst
}

function appendAutoinstall
{
	#1 is target to append
	#2 is item label
	#Make sure package exists on USB
	logMessage "Attempting to append autoinstall with ${1}.tar.gz"
	if [[ ! -e /fs/usb0/SyncMyRide/${1}.tar.gz ]]
	then
		logMessage "Missing package ${1}.tar.gz"
		corruptMessage
		safeExit
		exit 0
	fi
	LABEL="${2}"
	if [[ $LABEL = "" ]]
	then
		LABEL="$1"
	fi
	echo "Item${ITEM_INDEX} = ${LABEL}">>/fs/rwdata/storage/updater/autoinstall.lst
	echo "Open${ITEM_INDEX} = SyncMyRide\\${1}.tar.gz">>/fs/rwdata/storage/updater/autoinstall.lst
	ITEM_INDEX=$(( $ITEM_INDEX + 1 ))
	
}

function keyGenerator
{
	logMessage "Entering keyGenerator"
	touch /fs/rwdata/storage/updater/key.lst
	touch /fs/usb0/key.lst
	if [[ $vin = "" ]]
	then
		vin="XXXXXXXX"
	fi
	chmod 755 /fs/rwdata/storage/updater/universalUpdater
	/fs/rwdata/storage/updater/universalUpdater $vin
}

function upToDateCheck
{
	# $1 is the current module version number
	# $2 is the most up to date version number available for this module
	logMessage "Entering upToDateCheck"
	CURRENT_MAJOR=$(echo $1 | awk -F. '{ print $1 }')
	CURRENT_MINOR=$(echo $1 | awk -F. '{ print $2 }')
	CURRENT_SUFFIX=$(echo $1 | awk -F. '{ print $3 }')
	UP_TO_DATE_MAJOR=$(echo $2 | awk -F. '{ print $1 }')
	UP_TO_DATE_MINOR=$(echo $2 | awk -F. '{ print $2 }')
	UP_TO_DATE_SUFFIX=$(echo $2 | awk -F. '{ print $3 }')
	if [[ ( $CURRENT_MAJOR -gt $UP_TO_DATE_MAJOR ) || ( $CURRENT_MAJOR -eq $UP_TO_DATE_MAJOR && $CURRENT_MINOR -gt $UP_TO_DATE_MINOR ) || ( $CURRENT_MAJOR -eq $UP_TO_DATE_MAJOR && $CURRENT_MINOR -eq $UP_TO_DATE_MINOR && $CURRENT_SUFFIX -gt $UP_TO_DATE_SUFFIX ) ]]
	then
		logMessage "Module is currently running on newer version than what utility is updating"
		upToDateMessage
		safeExit
	fi
	if [[ $CURRENT_MAJOR -eq $UP_TO_DATE_MAJOR && $CURRENT_MINOR -eq $UP_TO_DATE_MINOR && $CURRENT_SUFFIX -eq $UP_TO_DATE_SUFFIX ]]
	then
		upToDateMessage
	fi
}

function determine
{
	#1 is hardware type
	#2 is screen size
	#3 is region
	#4 is country
	#5 is nav
	#6 is version
	logMessage "Entering getVehicleModel"
	chmod 755 /fs/rwdata/storage/updater/get_vin
	vin=$(/fs/rwdata/storage/updater/get_vin)
	if [[ $? -ne 0 ]]
	then
		logMessage "Could not collect VIN information"
		missingFileMessage
		safeExit
	fi
	logMessage "VIN: ${vin}"
	chmod 755 /fs/rwdata/storage/updater/get_model
	model=$(/fs/rwdata/storage/updater/get_model $vin)
	if [[ $? -ne 0 ]]
	then
		logMessage "Could not determine model from VIN: ${vin}"
		model="ALL"
	fi
	logMessage "Model: ${model}"
	HARDWARE=$1
	if [[ $HARDWARE = "$HW_OVERLAY" ]]
	then
		HARDWARE=$HW_30
	fi
	# look for country with specified model
	VERSION_NUMBER=$(grep "${HARDWARE}_${2}_${model}_${4}_${5}" /fs/rwdata/storage/updater/packages.lst | awk -F= '{ print $2 }')
	if [[ $VERSION_NUMBER = "" ]]
	then
		if [[ $model != "ALL" ]]
		then
			# country with specified model not present so search for catch all
			VERSION_NUMBER=$(grep "${HARDWARE}_${2}_ALL_${4}_${5}" /fs/rwdata/storage/updater/packages.lst | awk -F= '{ print $2 }')
			if [[ $VERSION_NUMBER != "" ]]
			then
				# search found when using ALL in place of model
				model="ALL"
			fi
		fi
	fi
	# country is not specified so look at region
	if [[ $VERSION_NUMBER = "" ]]
	then
		VERSION_NUMBER=$(grep "${HARDWARE}_${2}_${model}_${3}_${5}" /fs/rwdata/storage/updater/packages.lst | awk -F= '{ print $2 }')
	fi
	if [[ $VERSION_NUMBER = "" ]]
	then
		logMessage "Up to date version not detected for ${HARDWARE}_${2}_${model}_${3}_${5}"
		missingFileMessage
		safeExit
	fi
	if [[ $VERSION_NUMBER = "False" ]]
	then
		upToDateMessage
		safeExit
	fi
	upToDateCheck $6 $VERSION_NUMBER
	LIST_OF_PACKAGES=$(grep "${VERSION_NUMBER}_${HARDWARE}_${2}_${3}_${5}" /fs/rwdata/storage/updater/packages.lst)
	logMessage "Hardware: ${1}"
	logMessage "Screen size: ${2}"
	logMessage "Region: ${3}"
	logMessage "Country code: ${4}"
	logMessage "Navigation: ${5}"
	if [[ $LIST_OF_PACKAGES != "" ]]
	then
		generateAutoInstall "$1" "$2" "$3" "$5" "$VERSION_NUMBER"
	else
		logMessage "autoInstall packages not detected for ${VERSION_NUMBER}_${1}_${2}_${3}_${5}"
		missingFileMessage
	fi
	
}

function checkNav
{
	logMessage "Entering checkNav"
	NAVIGATION=$(df -h | awk '/hd0t180/ { sub("(\\.[0-9]+)?G$","",$2); print $2}')
	if [[ "$NAVIGATION" -lt 20 ]]
	then
		if [[ "$3" != "EU" ]]
		then
			determine "$1" "$2" "NEU" "$4" "$NNAV" "$5"
		fi
		determine "$1" "$2" "$3" "$4" "$NNAV" "$5"
	else
		determine "$1" "$2" "$3" "$4" "$NAV" "$5"
	fi
}

function checkRegion
{
	logMessage "Entering checkRegion"
	chmod 755 /fs/rwdata/storage/updater/get_region
	COUNTRY=$(grep "export COUNTRY" /tmp/ExportRegion.txt | awk -F= '{ print $2 }' | tr '[a-z'] '[A-Z]')
	REGION=$(/fs/rwdata/storage/updater/get_region $COUNTRY)
	RETURN_VALUE=$?
	# check if a cache file exists
	if [[ -e /fs/rwdata/storage/region.txt ]]
	then
		COUNTRY_CACHE=$(sed -n '1p' /fs/rwdata/storage/region.txt)
		REGION_CACHE=$(sed -n '2p' /fs/rwdata/storage/region.txt)

		# check if module codes and cached codes match
		if [[ "$COUNTRY" != "$COUNTRY_CACHE" || "$REGION" != "$REGION_CACHE" ]]
		then
			logMessage "Mismatch in country/region pair in cache. Module country: ${COUNTRY}, region: ${REGION}. Cached country: ${COUNTRY_CACHE}, region: ${REGION_CACHE}"
			COUNTRY="$COUNTRY_CACHE"
			REGION="$REGION_CACHE"
		fi
	else
		# codes were not cached
		if [[ $RETURN_VALUE -ne 0 ]]
		then
			# attempt to set country code on module and collect data again
			logMessage "Attempting to set country code"
			/fs/mp/apps/SS_SetRegion
			COUNTRY=$(grep "export COUNTRY" /tmp/ExportRegion.txt | awk -F= '{ print $2 }' | tr '[a-z'] '[A-Z]')
			REGION=$(/fs/rwdata/storage/updater/get_region $COUNTRY)
			if [[ $? -ne 0 ]]
			then
				# region could not be detected and there are no codes cached
				logMessage "Could not detect region based off of code: ${COUNTRY}"
				missingFileMessage
				safeExit
			fi
		fi
	fi
	echo "$COUNTRY">/fs/rwdata/storage/region.txt
	echo "$REGION">>/fs/rwdata/storage/region.txt
	checkNav "$1" "$2" "$REGION" "$COUNTRY" "$3"
}

function checkGT
{
	if [[ $1 = "2.3.17179" ]]
	then
		upToDateMessage
	fi
}

function MY16_Module
{
	# $1 is version number
	COUNTRY=$(grep "export COUNTRY" /tmp/ExportRegion.txt | awk -F= '{ print $2 }')
	MAJOR=$(echo $1 | awk -F. '{ print $1 }')
	if [[ $MAJOR -eq 1 ]]
	then
		checkNav "$HW_30" "6" "NA" "$COUNTRY" "$1"
	fi
}

function B460Check
{
	logMessage "Entering B460Check"
	# This function is to check if the module is a B460 meaning it is 3.1.1 and is compatible with my2019 software
	# Parameter $1 will always be 3.1 or 3.1.1
	# $2 is the version number we are running
	MAJOR=$(echo $2 | awk -F. '{ print $1 }')
	MINOR=$(echo $2 | awk -F. '{ print $2 }')
	
	MAJOR_MY18=3
	MINOR_MY18=0
	OUTLIER=213
	
	# If newer than 2018
	if [[ $MAJOR -eq $MAJOR_MY18 && $MINOR -ne $MINOR_MY18 && $MINOR -ne $OUTLIER ]]
	then
		# We have a B460 module
		screenCheck "$HW_B460" "$2"
	else
		# not a B460 module
		screenCheck "$1" "$2"
	fi
}

function checkOverlay
{
	logMessage "Entering checkOverlay"
	#1 is version
	if [[ $1 = "2.0.16074" || $1 = "2.1.16211" || $1 = "2.1.16116" || $1 = "2.1.16217" || $1 = "2.2.16280" || $1 = "2.2.17011" || $1 = "2.1.16173" || $1 = "2.2.16223" || $1 = "2.3.16350" ]]
	then
		checkRegion "$HW_OVERLAY" "6" "$1"
		safeExit
	fi
	MA=$(echo $1 | awk -F. '{ print $1 }')
	MI=$(echo $1 | awk -F. '{ print $2 }')
	if [[ $MA -eq 2 && $MI -lt 2 ]]
	then
		checkRegion "$HW_30" "6" "$1"
		safeExit
	fi
}

function hwCheck
{
	logMessage "Hardware variant: ${MY}"
	# $1 is the version number
	if [[ "$MY" -eq 16 ]]
	then
		# we have a 3.0 module
		screenCheck "$HW_30" "$1"
	fi
	if [[ "$MY" -eq 17 ]]
	then
		# we have a 3.1 or 3.1.1 module
		# we pass HW_30 because 3.0, 3.1, and 3.1.1 modules have the same autoinstall.lst
		B460Check "$HW_31" "$1"
	fi
	if [[ "$MY" -eq 1850 ]]
	then
		# we have a 3.2 module
		screenCheck "$HW_32" "$1"
	fi
}

function screenCheck
{
	# $1 is the hardware version we are running
	# $2 is the version number we are running
	# 2.1.16217 and ealrier do not work
	
	logMessage "Entering screenCheck"
	chmod 755 /fs/rwdata/storage/updater/get_display
	/fs/rwdata/storage/updater/get_display
	SCREEN=$?
	logMessage "Screen enum detected: ${SCREEN}"
	if [[ $SCREEN -lt 2 || $SCREEN -gt 22 ]]
	then
		screenDetectionMessage
	fi
	# 5, 6 is 4" screen
	if [[ $SCREEN -eq 5 || $SCREEN -eq 6 ]]
	then
		SCREEN_SIZE=4
	fi
	# 4, 7, 12, 14, 15
	if [[ $SCREEN -eq 4 || $SCREEN -eq 7 || $SCREEN -eq 12 || $SCREEN -eq 14 || $SCREEN -eq 15 ]]
	then
		SCREEN_SIZE=6
	fi
	# 2, 3, 8, 11, 13, 16, 17, 18, 19, 20 is a 6/8" screen
	if [[ $SCREEN -eq 2 || $SCREEN -eq 3 || $SCREEN -eq 8 || $SCREEN -eq 11 || $SCREEN -eq 13 || ( $SCREEN -gt 15 && $SCREEN -lt 21 ) ]]
	then
		SCREEN_SIZE=8
	fi
	# 9, 10, 21, 22 is 10" screen
	if [[ $SCREEN -eq 9 || $SCREEN -eq 10 || $SCREEN -eq 21 || $SCREEN -eq 22 ]]
	then
		SCREEN_SIZE=10
	fi
	logMessage "Screen size detected: ${SCREEN_SIZE}"
	checkRegion "$1" "$SCREEN_SIZE" "$2"
}

function checkKey
{
	logMessage "Checking keys"
	read sync3Key < /fs/rwdata/storage/updater/key.lst
	read usbKey < /fs/usb0/key.lst
	cmp -s /fs/usb0/autoinstall.lst /fs/rwdata/storage/updater/autoinstall.lst
	BASE_AUTOINSTALL=$?
	
	# keys match, they are not empty, and stored autoinstall.lst matches the usb root autoinstall.lst
	if [[ "$sync3Key" = "$usbKey" && "$sync3Key" != "" && $BASE_AUTOINSTALL -eq 0 ]]
	then
		logMessage "Keys match"
		INDICATOR=$(find /fs/images/ivsu_cache -maxdepth 1 -name "*.trans" | wc -l)
		# checks if we are on the first part of a transaction so we don't have to sleep 5 every time
		if [[ $INDICATOR -gt 0 ]]
		then
			logMessage "Entering First part of transaction"
			sh /fs/rwdata/storage/updater/status_message.sh $UPDATE_ARGUMENT $TRANSACTION_ONE &
			sleep 5
		else
			logMessage "Entering final phase of update"
			sh /fs/rwdata/storage/updater/status_message.sh $UPDATE_ARGUMENT $TRANSACTION_TWO &
		fi
		exit 0
	else
		# Keys don't match or they are empty
		invalidKey
		shutdown
	fi
}

function leaseCheck
{
	logMessage "Entering leaseCheck"
	logMessage "Date on vehicle: $NOW"
	# check to see if lease file exists
	if [[ ! -e /fs/usb0/SyncMyRide/autoInstalls/Lease.txt ]]
	then
		logMessage "Lease.txt not present on usb"
		leaseAbsentMessage
		safeExit
	fi
	read usbLease < /fs/usb0/SyncMyRide/autoInstalls/Lease.txt
	usbLease=$(echo $usbLease | tr -d '\r')
	logMessage "USB lease code: $usbLease"
	chmod 755 /fs/rwdata/storage/updater/get_lease
	LEASERETURNVALUE=$(/fs/rwdata/storage/updater/get_lease $usbLease)
	if [[ $? -ne 0 ]]
	then
		logMessage "leaseCheck error"
		leaseAbsentMessage
		safeExit
	fi

	# Lease is in the acceptable range
	if [[ LEASERETURNVALUE -eq 0 ]]
	then
		return
	fi
	# Lease is in the acceptable range but about to expire
	if [[ LEASERETURNVALUE -eq 1 ]]
	then
		leaseAlmostExpiredMessage
		sleep 7
		return
	fi
	# Expired lease code
	if [[ LEASERETURNVALUE -eq 2 ]]
	then
		logMessage "USB lease code expired"
		leaseExpiredMessage
	fi
	# invalid lease code
	if [[ LEASERETURNVALUE -eq 3 ]]
	then
		logMessage "USB lease code not valid"
		leaseExpiredMessage
	fi
	# Incorrect module date
	if [[ LEASERETURNVALUE -eq 4 ]]
	then
		logMessage "Detected module date needs to be updated"
		return
	fi
 
}

function fileCheck
{
	# First check to see if the file exists
	if [[ -e /fs/rwdata/storage/updater/key.lst && -e /fs/usb0/key.lst ]]
	then
		checkKey
		
	# File doesn't exist so we create a directory and create a file in the directory
	else
		invalidKey
		shutdown
	fi
}

function verificationCheck
{
	logMessage "Entering verificationCheck"
	# check to see if cookie file exists
	if [[ -e /fs/usb0/SyncMyRide/autoInstalls/30_6_NNA_NNAV.lst ]]
	then
		fileCheck
	else
		logMessage "Missing 30_6_NNA_NNAV.lst"
		extractTar
		corruptMessage
	fi
}
#####################################SCRIPT START#######################################################
logMessage "STARTING SCRIPT"
getLanguage
verificationCheck

exit 0